@@ -1,7 +1,8 @@
 import requests,datetime,time,warnings,json,calendar,math,os
 from datetime import date
-# from dateutil.relativedelta import relativedelta
+from dateutil.relativedelta import relativedelta
 from pandas.io.json import json_normalize
+from urllib.parse import quote
 import pandas as pd
 import numpy as np
 from scipy.stats import norm, mstats
@@ -14,9 +15,9 @@ warnings.filterwarnings("ignore")
 2. 计算七麦内的备用工具；
 3. 常用的自定义参数；
 4. 获取基础信息相关数据；
-5. 获取榜单相关数据；
-6. 获取开发商相关数据；
-7. 获取关键词的相关数据；
+5. 获取产品榜单相关数据；
+6. 获取产品开发商相关数据；
+7. 获取产品关键词的相关数据；
 8. 获取产品的关键词相关数据；
 9. 获取产品评论的相关接口；
 10. 获取清榜列表相关数据；
@@ -216,6 +217,81 @@ class Qimai_Outside_Tool:
             self.trend = '无趋势', p, g, z
         return self.trend
 
+    def main_kendall_Trend(self, column_name):
+        # 读取数据；
+        run_value_dataframe = pd.DataFrame({})  # 使用dataframe存储数据；
+        # df['日期'] = pd.to_datetime(df['日期'])
+        df = self.data_info[0]
+        df['日期'] = [datetime.date.fromisoformat(str(i)[:10]) for i in df['日期'].values]
+        end_time = max(df['日期'].values)
+        for i in range(30, 91):
+            start_time = min(df['日期'].values)
+            run_days = datetime.timedelta(days=i)
+            while start_time <= end_time and (end_time - start_time) > run_days:
+                run_time = start_time + run_days
+                run_df = df[(df['日期'] >= start_time) & (df['日期'] <= run_time)][column_name]
+                list_c = [int(i) for i in run_df.values]
+                thrend_str = Qimai_Outside_Tool(list_c).trend_analysis()
+                # if thrend_str[0] == '上升':
+                # print('===========打印【%s】天的运行数据===========' %(run_days.days))
+                run_value_dataframe = run_value_dataframe.append(pd.DataFrame({
+                    'start_time': [start_time],
+                    'end_time': [run_time],
+                    'run_days': [run_days],
+                    'thrend_str': [thrend_str[0]],
+                    'p_value': [thrend_str[1]],
+                    'g_value': [thrend_str[2]],
+                    'z_value': [thrend_str[3]]
+                }))
+
+                # 时间往前加；
+                start_time += run_days
+
+        ##################【方案一 - 根据涨幅挑选2-3个区间】##################
+        # 先排序，再根据排序后的值筛选出几个，需要避免重叠；
+        trend_value_list = ['上升', '下降', '无趋势']
+        df_dataframe_all = pd.DataFrame({})
+        for trend_value in trend_value_list:
+            run_value_now = run_value_dataframe[run_value_dataframe['thrend_str']==trend_value].sort_values('z_value', ascending=False)
+            run_value_list = [list(i) for i in run_value_now.values]
+            run_i = 0  # 便于重复检测；
+            for i in run_value_list:
+                if run_i + 1 < len(run_value_list):
+                    run_x = 0
+                    while True:
+                        if run_x + 1 <= len(run_value_list):
+                            # overlap, s1_scale, s2_scale = calc_overlap_days(i[0],i[1],x[0],x[1])
+                            overlap, s1_scale, s2_scale = Qimai_Outside_Tool().calc_overlap_days(run_value_list[run_i][0], run_value_list[run_i][1], run_value_list[run_x][0], run_value_list[run_x][1])
+                            if overlap > 0:
+                                if run_value_list[run_i][6] > run_value_list[run_x][6]:  # 说明有重叠，保留前面的；
+                                    run_value_list.remove(run_value_list[run_x])
+                                    run_x -= 1
+                                elif run_value_list[run_i][6] < run_value_list[run_x][6]:
+                                    run_value_list.remove(run_value_list[run_i])
+                                    run_i -= 1
+                                    break
+                                else:
+                                    if run_value_list[run_i][2] < run_value_list[run_x][2]:
+                                        run_value_list.remove(run_value_list[run_x])
+                                        run_x -= 1
+                                    elif run_value_list[run_i][2] > run_value_list[run_x][2]:
+                                        run_value_list.remove(run_value_list[run_x])
+                                        run_i -= 1
+                                        break
+                            run_x += 1
+                        else:
+                            # print('跳出', print('即将跳出', run_x, len(run_value_list)))
+                            break
+                    run_i += 1
+                else:
+                    break
+
+            df_dataframe = pd.DataFrame(run_value_list)
+            df_dataframe_all = pd.concat([df_dataframe_all, df_dataframe])
+        df_dataframe_all.columns = ['开始时间', '结束时间', '持续天数', '趋势', 'p值', 'g值', 'z值']
+        # return df_dataframe[['开始时间', '结束时间', '持续天数', 'z值']]
+        return df_dataframe_all
+
 # 计算七麦内的备用工具；
 class Qimai_Intside_Tool:
     def __init__(self, data_info):
@@ -386,6 +462,17 @@ class Get_Keyword_Info(Qimai_Diy_Var):
         self.keyword_WordInfo = res.json()
         return self.keyword_WordInfo
 
+    def get_keywordHistory_hints(self, start_date, end_date):
+        url = 'https://api.qimai.cn/app/searchHints'
+        payload='device=%s&word[0]=%s&country=%s&sdate=%s&edate=%s' %(self.device, quote(self.keyword, 'utf-8'), self.country, start_date, end_date)
+        headers = {
+            'Content-Type': 'application/x-www-form-urlencoded',
+            "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/78.0.3904.87 Safari/537.36"
+        }
+        res = session.post(url, data=payload, headers=headers)
+        self.keywordHistory_hints = res.json()
+        return self.keywordHistory_hints
+
     def get_top_to_df(self, top_num=100):
         self.get_keyword_search()
         keyword_top_list = []
@@ -761,7 +848,7 @@ class Get_FreePaidGross_RankList(Qimai_Diy_Var):
         res = session.get(url, headers=headers)
         self.rank_update_List = res.json()
 
-    def get_freeRank_list(self):
+    def get_freeRank_list(self, max_index=1000000):
         self.app_freeRank_list = []
         page_num = 1
         while True:
@@ -769,11 +856,12 @@ class Get_FreePaidGross_RankList(Qimai_Diy_Var):
             res = session.get(url, headers=headers)
             self.app_freeRank_list.append(res.json())
             page_num += 1
-            if page_num > res.json()['maxPage']:
+            now_one_index = res.json()['rankInfo'][0]['index']
+            if page_num > res.json()['maxPage'] or now_one_index <= max_index:
                 break
         return self.app_freeRank_list
 
-    def get_paidRank_list(self):
+    def get_paidRank_list(self, max_index=1000000):
         self.app_paidRank_list = []
         page_num = 1
         while True:
@@ -781,11 +869,12 @@ class Get_FreePaidGross_RankList(Qimai_Diy_Var):
             res = session.get(url, headers=headers)
             self.app_paidRank_list.append(res.json())
             page_num += 1
-            if page_num > res.json()['maxPage']:
+            now_one_index = res.json()['rankInfo'][0]['index']
+            if page_num > res.json()['maxPage'] or now_one_index <= max_index:
                 break
         return self.app_paidRank_list
 
-    def get_grossRank_list(self):
+    def get_grossRank_list(self, max_index=1000000):
         self.app_grossRank_list = []
         page_num = 1
         while True:
@@ -793,7 +882,8 @@ class Get_FreePaidGross_RankList(Qimai_Diy_Var):
             res = session.get(url, headers=headers)
             self.app_grossRank_list.append(res.json())
             page_num += 1
-            if page_num > res.json()['maxPage']:
+            now_one_index = res.json()['rankInfo'][0]['index']
+            if page_num > res.json()['maxPage'] or now_one_index <= max_index:
                 break
         return self.app_grossRank_list
 
